<?php
require __DIR__ . '/app/bootstrap.php';
$bootstrap = \Magento\Framework\App\Bootstrap::create(BP, $_SERVER);
$objectManager = $bootstrap->getObjectManager();
$appState = $objectManager->get('\Magento\Framework\App\State');
$appState->setAreaCode(\Magento\Framework\App\Area::AREA_CRONTAB);
// author: fabrizio vanzani
class script_session_migration
{
    protected $objectManager;
    protected $methods=['db2files','files2db'];
    protected $method;
    protected $modes=['insert','replace']; // todo: upsert
    protected $mode='insert';
    protected $dryrun=true;
    protected $filterSessionId;

    public function __construct()
    {
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    }

    public function setMethod(String $method)
    {
        $this->method=$method;
    }
    public function setMode(String $mode)
    {
        $this->mode=$mode;
    }
    public function setDryrun(Bool $dryrun=true)
    {
        $this->dryrun=$dryrun;
    }
    public function setFilterSessionId(String $sessionId)
    {
        $this->filterSessionId=$sessionId;
    }
    protected function getMethod()
    {
        if (!in_array($this->method,$this->methods)) {
            throw new Exception("Migration Direction '{$this->method}' not found.");
        }
        return $this->method;
    }
    protected function getMode()
    {
        if (!in_array($this->mode,$this->modes)) {
            throw new Exception("Migration Mode '{$this->mode}' not found.");
        }
        return $this->mode;
    }
    public function execute()
    {
        echo "Migraiton session data with method {$this->getMethod()} and mode {$this->getMode()} \n";
        $this->{$this->getMethod()}();

    }
    protected function db2files()
    {   //
        $objectManager=$this->objectManager;
        $directory = $objectManager->get('\Magento\Framework\Filesystem\DirectoryList');
        $sessionDir=$directory->getPath('session').'/';
        $mode=$this->getMode();
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('session');
        $sql = "Select * FROM " . $tableName ;
        if ($this->filterSessionId) {
            $sql .= " WHERE session_id = '{$this->filterSessionId}'";
        }
        $sql .= " order by session_expires DESC ";

        echo "$sql\n";
        $result = $connection->fetchAll($sql);
        foreach ($result as $item) {
            //$item= $result[0];
            $sessionId=$item['session_id'];
            $serializedData=$item['session_data'];
            $sessionExpires=$item['session_expires'];
            $decodedData = base64_decode($serializedData, true);
            $data = null;
            if ($decodedData !== false) {
                $data = $decodedData;
            }
            //  $sessionData = base64_encode($sessionData);
            //print_r($data);
            $sessionFile=$sessionDir.'sess_'.$sessionId;
            $msgMode=$msgSuccess='';
            $fileExists=is_file($sessionFile);

            $doWrite=($mode=='replace' || ($mode=='insert' && !$fileExists ) ) && !$this->dryrun;

            if ($this->filterSessionId && $fileExists) {
                if ($data==file_get_contents($sessionFile)) {
                    echo "Comparison: SAME CONTENT\n";
                } else {
                    echo "Comparison:\n";
                    print_r($data);
                    echo "\n";
                    print_r(file_get_contents($sessionFile));
                    echo "\n";
                }
            }

            if ($doWrite) {
                if(file_put_contents($sessionFile,$data)) {
                    $writeStr='Write Success';
                } else {
                    $writeStr='Write Error  ';
                }
                touch($sessionFile, $sessionExpires ); // todo: test
            } else {
                $writeStr='Do not write ***';
            }

            $fileExistsStr=$fileExists?"    File Exists":"File Not Exists";
            $simulationStr=$this->dryrun?"(simulation)":"";
            echo "SessionId $sessionId expires=$sessionExpires mode=$mode  $fileExistsStr  $writeStr  $simulationStr\n";
        }


    }

    protected function files2db()
    {
        $objectManager=$this->objectManager;
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('session');
        $mode=$this->getMode();
        $directory = $objectManager->get('\Magento\Framework\Filesystem\DirectoryList');
        $sessionDir=$directory->getPath('session').'/';
        $pattern=$sessionDir."sess*";
        if ($this->filterSessionId) {
            $pattern=$sessionDir."sess_{$this->filterSessionId}";
            if (!is_file($pattern)) {
                echo "SessionId {$this->filterSessionId} file not found\n";
            }
        }
        foreach(glob($pattern) as $sessionFile)  {
            //echo "$sessionFile \n";
            if(!preg_match('/sess_(.*)$/',$sessionFile,$m)) {
                continue;
            }
            $sessionId=$m[1];
            $sessionExpires=filemtime($sessionFile);
            $sessionData=file_get_contents($sessionFile);
            $sessionData = base64_encode($sessionData);

            $sql = "Select * FROM " . $tableName ." WHERE session_id='$sessionId'";
            $result = $connection->fetchRow($sql);
            $dbExists= (bool) $result;

            $doWrite=($mode=='replace' || ($mode=='insert' && !$dbExists ) ) && !$this->dryrun;

            if ($this->filterSessionId && $dbExists) {
                if ($sessionData==$result['session_data']) {
                    echo "Comparison: SAME CONTENT\n";
                } else {
                    echo "Comparison:\n";
                    print_r($sessionData);
                    echo "\n";
                    print_r($result['session_data']);
                    echo "\n";
                }
            }

            if ($doWrite ) {
                $sql = "replace into " . $tableName . " (session_id, session_expires, session_data) Values ('$sessionId','$sessionExpires','$sessionData')";
                //echo $sql."\n";
                $result=$connection->query($sql);
                if($result) {
                    $writeStr='Write Success';
                } else {
                    $writeStr='Write Error  ';
                }
            } else {
                $writeStr='Do not write ***';
            }

            $dbExistsStr=$dbExists?"    Db Exists":"Db Not Exists";
            $simulationStr=$this->dryrun?"(simulation)":"";
            echo "SessionId $sessionId expires=$sessionExpires mode=$mode  $dbExistsStr  $writeStr  $simulationStr\n";

        }
    }

}
if (!isset($argv[3])) {
    echo "Usage:\n";
    $self=basename(__FILE__);
    echo "php $self <method> <mode> <dryrun|save> [session_id] \n";
    echo "methods: db2files, files2db \n";
    echo "modes: insert, replace, upsert (todo) \n";
    echo "Examples:\n";
    echo "php $self db2files insert save  \n";
    echo "php $self db2files insert dryrun  \n";
    echo "php $self db2files insert save fefa51f32efe28770ba23401aa1769fd \n";
    echo "php $self files2db insert save  \n";
    echo "php $self files2db replace dryrun  \n";
    echo "php $self files2db insert save fefa51f32efe28770ba23401aa1769fd \n";
    exit;
}


$job=$objectManager->get('\script_session_migration');
if (isset($argv[1]))
    $job->setMethod($argv[1]);
if (isset($argv[2]))
    $job->setMode($argv[2]);
if (isset($argv[3]))
    $job->setDryrun($argv[3]=='save'?false:true);
if (isset($argv[4]))
    $job->setFilterSessionId($argv[4]);

$job->execute();